==> plasma-desktop-5.2.0.tar.xz: plasma-desktop-5.2.0/applets/taskmanager/plugin/draghelper.cpp <==
/***************************************************************************
 *   Copyright (C) 2013 by Eike Hein <hein@kde.org>                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "draghelper.h"

#include <QApplication>
#include <QDrag>
#include <QMimeData>
#include <QQuickItem>
#include <QIcon>
#include <QPixmap>

DragHelper::DragHelper(QObject* parent) : QObject(parent)
{
}

DragHelper::~DragHelper()
{
}

bool DragHelper::isDrag(int oldX, int oldY, int newX, int newY) const
{
    return ((QPoint(oldX, oldY) - QPoint(newX, newY)).manhattanLength() >= QApplication::startDragDistance());
}

void DragHelper::startDrag(QQuickItem *item, const QString &mimeType,
    const QVariant &mimeData, const QUrl &url, const QIcon &icon) const
{
    QList<QUrl> urlList;
    urlList.append(url);

    QMimeData *dragData = new QMimeData();
    dragData->setData(mimeType, mimeData.toByteArray());
    dragData->setUrls(urlList);

    QDrag *drag = new QDrag(item);
    drag->setMimeData(dragData);
    drag->setPixmap(icon.pixmap(QSize(48, 48)));
    drag->setHotSpot(QPoint(drag->pixmap().width() / 2, drag->pixmap().height() / 2));

    item->grabMouse();
    drag->exec();
    item->ungrabMouse();

    emit dropped();
}

==> plasma-desktop-5.3.0.tar.xz: plasma-desktop-5.3.0/applets/taskmanager/plugin/draghelper.cpp <==
/***************************************************************************
 *   Copyright (C) 2013 by Eike Hein <hein@kde.org>                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "draghelper.h"

#include <QApplication>
#include <QDrag>
#include <QMimeData>
#include <QQuickItem>
#include <QIcon>
#include <QPixmap>

DragHelper::DragHelper(QObject* parent) : QObject(parent)
{
}

DragHelper::~DragHelper()
{
}

bool DragHelper::isDrag(int oldX, int oldY, int newX, int newY) const
{
    return ((QPoint(oldX, oldY) - QPoint(newX, newY)).manhattanLength() >= QApplication::startDragDistance());
}

void DragHelper::startDrag(QQuickItem *item, const QString &mimeType,
    const QVariant &mimeData, const QUrl &url, const QIcon &icon)
{
    QMetaObject::invokeMethod(this, "startDragInternal", Qt::QueuedConnection,
        Q_ARG(QQuickItem*, item),
        Q_ARG(QString, mimeType),
        Q_ARG(QVariant, mimeData),
        Q_ARG(QUrl, url),
        Q_ARG(QIcon, icon));
}

void DragHelper::startDragInternal(QQuickItem *item, const QString &mimeType,
    const QVariant &mimeData, const QUrl &url, const QIcon &icon) const
{
    QList<QUrl> urlList;
    urlList.append(url);

    QMimeData *dragData = new QMimeData();
    dragData->setData(mimeType, mimeData.toByteArray());
    dragData->setUrls(urlList);

    QDrag *drag = new QDrag(item);
    drag->setMimeData(dragData);
    drag->setPixmap(icon.pixmap(QSize(48, 48)));
    drag->setHotSpot(QPoint(drag->pixmap().width() / 2, drag->pixmap().height() / 2));

    item->grabMouse();
    drag->exec();
    item->ungrabMouse();

    emit dropped();
}
